@@ -406,6 +406,11 @@ impl View for ArticleView {
             Event::Key(Key::Up) => self.move_focus(Directions::UP),
             Event::Key(Key::Down) => self.move_focus(Directions::DOWN),
             Event::Key(Key::Enter) => {
+                if self.content.link_handler.links.len() >= self.content.link_handler.current_link {
+                    log::error!("Failed trying to access an invalid link");
+                    return EventResult::Consumed(None);
+                }
+
                 let target = self.content.link_handler.links
                     [self.content.link_handler.current_link]
                     .destination
